from packaging.version import Version
from pathlib import Path
import os
import subprocess
import sys


APP_HOME = os.environ.get('TAKARA_COGENTIP_HOME')
if APP_HOME is None:
    APP_HOME = Path(os.path.realpath(os.path.dirname(__file__))).parents[0]
elif not os.path.isdir(APP_HOME):
    print(f"The app's home directory at \"{APP_HOME}\" does not exist. Please double check and export the value of the TAKARA_COGENTIP_HOME environment variable.")
    sys.exit(1)
else:
    APP_HOME = Path(APP_HOME)

MINIMUM_TRUST4_VERSION = '1.1.4'


def _is_trust4_version_unavailable(trust4_version):
    devnull = open(os.devnull, 'w')
    return_code = subprocess.call(['conda', 'search', '-f', f'bioconda::trust4={trust4_version}'], stdout=devnull, stderr=devnull)
    if return_code:
        return True
    else:
        return False

def _is_trust4_version_unsupported(trust4_version):
    return Version(trust4_version) < Version(MINIMUM_TRUST4_VERSION)

def check_trust4_version(trust4_version):
    if _is_trust4_version_unavailable(trust4_version):
        die(f'TRUST4 version not available: {trust4_version}')
    if _is_trust4_version_unsupported(trust4_version):
        die(f"""Unsupported TRUST4 version selected: {trust4_version}
Minimum TRUST4 version required: {MINIMUM_TRUST4_VERSION}
""")

def die(reason, code=1):
  print(f'ERROR: {reason}')
  sys.exit(code)

def get_version():
    with open(os.path.join(APP_HOME, "VERSION")) as f:
        return f.read().strip()
