import java.nio.file.Paths

params.species_common = params.species

if (params.species_common == "mouse") {
  params.species_taxonomic = "mus_musculus"
} else if (params.species_common == "human") {
  params.species_taxonomic = "homo_sapiens"
} else {
  error("Unknown species: " + params.species_common)
}

species_fa = Paths.get(projectDir.toString(), "share", "imgt", "IMGT_" + params.species_taxonomic + ".fa").toAbsolutePath()

uses_umi_map = [
  'homo_sapiens': [
    'TCRv1': false,
    'TCRv2': true,
    'BCRv1': true,
    'BCRv2': true,
  ],
  'mus_musculus': [
    'TCRv1': false,
    'TCRv2': true,
    'BCRv1': false,
    'BCRv2': true,
  ],
]
params.kit_uses_umi = uses_umi_map[params.species_taxonomic][params.receptor_kit]

unusual_gaps_map = [
  'homo_sapiens': [
    'TCRv1': '',
    'TCRv2': '',
    'BCRv1': '',
    'BCRv2': '',
  ],
  'mus_musculus': [
    'TCRv1': 'TRAV:7,83',
    'TCRv2': 'TRAV:7,83',
    'BCRv1': '',
    'BCRv2': '',
  ],
]
params.unusual_gaps = unusual_gaps_map[params.species_taxonomic][params.receptor_kit]

include { SPLIT_METADATA_BY_SAMPLE } from './modules/split_metadata_by_sample'
include { PREPROCESS } from './modules/preprocess'
include { SPLIT_FASTQ_FILES } from './modules/split_fastq_files'
include { PLOT_UMI_GROUP_FREQUENCIES } from './modules/plot_umi_group_frequencies'
include { TRUST4 } from './modules/trust4'
include { MERGE_TRUST4_CHUNKS as MERGE_AIRRTSV_FROM_TRUST4_CHUNKS } from './modules/merge_trust4_chunks'
include { MERGE_TRUST4_CHUNKS as MERGE_AIRRALIGNTSV_FROM_TRUST4_CHUNKS } from './modules/merge_trust4_chunks'
include { MERGE_TRUST4_CHUNKS as MERGE_ANNOT_FROM_TRUST4_CHUNKS } from './modules/merge_trust4_chunks'
include { MERGE_TRUST4_CHUNKS as MERGE_ASSEMBLEDREADSFA_FROM_TRUST4_CHUNKS } from './modules/merge_trust4_chunks'
include { MERGE_TRUST4_CHUNKS as MERGE_BARCODEAIRRTSV_FROM_TRUST4_CHUNKS } from './modules/merge_trust4_chunks'
include { MERGE_TRUST4_CHUNKS as MERGE_BARCODEREPORTTSV_FROM_TRUST4_CHUNKS } from './modules/merge_trust4_chunks'
include { MERGE_TRUST4_CHUNKS as MERGE_CDR3OUT_FROM_TRUST4_CHUNKS } from './modules/merge_trust4_chunks'
include { MERGE_TRUST4_CHUNKS as MERGE_FINALOUT_FROM_TRUST4_CHUNKS } from './modules/merge_trust4_chunks'
include { MERGE_TRUST4_CHUNKS as MERGE_RAWOUT_FROM_TRUST4_CHUNKS } from './modules/merge_trust4_chunks'
include { MERGE_TRUST4_CHUNKS as MERGE_REPORTTSV_FROM_TRUST4_CHUNKS } from './modules/merge_trust4_chunks'
include { MERGE_TRUST4_CHUNKS as MERGE_TOASSEMBLE1FQ_TRUST4_CHUNKS } from './modules/merge_trust4_chunks'
include { MERGE_TRUST4_CHUNKS as MERGE_TOASSEMBLE2FQ_FROM_TRUST4_CHUNKS } from './modules/merge_trust4_chunks'
include { MERGE_TRUST4_CHUNKS as MERGE_TOASSEMBLEFA_TRUST4_CHUNKS } from './modules/merge_trust4_chunks'
include { GENERATE_FINAL_REPORTS } from './modules/generate_final_reports'

workflow {
  meta_file = new File(params.meta_file)
  meta_file_dir = file(meta_file.absolutePath).parent

  ch_split_metadata_by_sample_args = Channel.fromPath(meta_file)
    .splitCsv( header:true )
    .map { row ->
      is_read1_file_name_absolute_path = row.read1_file_name.startsWith('/')
      is_read2_file_name_absolute_path = row.read2_file_name.startsWith('/')
      [meta_file_dir, row.sampleID, row.read1_file_name, row.read2_file_name, is_read1_file_name_absolute_path, is_read2_file_name_absolute_path]
    }

  ch_preprocess_args =
    (ch_split_metadata_by_sample_args | SPLIT_METADATA_BY_SAMPLE)
    .map { meta ->
      sample = meta[0]
      split_metadata_file = meta[1]
      tuple(sample, split_metadata_file)
    }

  ch_preprocess_args | PREPROCESS

  PREPROCESS.out.qc_stats
    .collectFile(name: "sample_QC_stats.csv", keepHeader: true, sort: { it.text }, storeDir: "${params.outdir}/preprocess")

  PREPROCESS.out.qc_stats_airr
    .collectFile(name: "sample_QC_stats.airr.csv", keepHeader: true, sort: { it.text }, storeDir: "${params.outdir}/preprocess")

  ch_split_fastq_files_args =
    PREPROCESS.out.fastq
    .map { input ->
      sample_passed = input[0]
      map =
        input[1]
        .groupBy { fname ->
          sample_parsed = fname.baseName.split('_')[0]
          isotype = fname.baseName.split('_')[1]
          if (sample_passed != sample_parsed) {
            error("ERROR: Sample ID ('" + sample_passed + "') passed from PREPROCESS does not match Sample ID ('"
              + sample_parsed + "') parsed from FQ file name: " + fname
            )
          }
          tuple(sample_parsed, isotype)
        }
        .findAll {
          isSupportedIsotype =
            params.receptor_kit == "TCRv1"
              ? it.key[1] =~ /TR[A|B]/
              : params.receptor_kit == "TCRv2"
              ? it.key[1] =~ /TR[A|B]/
              : params.receptor_kit == "BCRv1"
              ? it.key[1] =~ /IG[G|K|L|M]/
              : params.receptor_kit == "BCRv2"
              ? it.key[1] =~ /IG[A|D|E|G|K|L|M]/
              : false

          isNotEmpty = it.value.every { path -> !path.isEmpty() }

          isSupportedIsotype && isNotEmpty
        }

      map
    }
    .flatMap { sample_map ->
      map = sample_map
      def keysToRemove = []
      map.forEach( (k, v) -> {
        if (v.size() < 2) {
          keysToRemove.add(k)
        }
      })
      for(key in keysToRemove) {
        map.remove(key)
      }
      map.collect { sample_isotype, f1f2 ->
        tuple(sample_isotype[0], sample_isotype[1], f1f2[0], f1f2[1])
      }
    }

  ch_split_fastq_files_args | SPLIT_FASTQ_FILES

  SPLIT_FASTQ_FILES.out.umi_group_sizes_frequency
    .groupTuple() | PLOT_UMI_GROUP_FREQUENCIES

  ch_trust4_args =
    SPLIT_FASTQ_FILES.out.fastq
    .map { sample, isotype, f1f2, logs ->
      chunk_f1f2_map =
        f1f2
        .groupBy { fname ->
          chunk = fname.baseName.split('_')[1]
        }
      tuple(sample, isotype, chunk_f1f2_map)
    }
    .flatMap { sample, isotype, chunk_f1f2_map ->
      chunk_f1f2_map.collect { chunk, f1f2 ->
        tuple(sample, isotype, chunk, f1f2[0], f1f2[1], species_fa) }
    }

  ch_trust4 = ch_trust4_args | TRUST4

  airr_align_tsv = TRUST4.out.airr_align_tsv
    .groupTuple(by: [0, 1, 2, 3])
    .map { sample, isotype, has_header, filename, chunk_files ->
      tuple(sample, isotype, has_header, filename, chunk_files.flatten())
    }
    | MERGE_AIRRALIGNTSV_FROM_TRUST4_CHUNKS
    
  annot_fa = TRUST4.out.annot_fa
    .groupTuple(by: [0, 1, 2, 3])
    .map { sample, isotype, has_header, filename, chunk_files ->
      tuple(sample, isotype, has_header, filename, chunk_files.flatten())
    }
    | MERGE_ANNOT_FROM_TRUST4_CHUNKS

  cdr3_out = TRUST4.out.cdr3_out
    .groupTuple(by: [0, 1, 2, 3])
    .map { sample, isotype, has_header, filename, chunk_files ->
      tuple(sample, isotype, has_header, filename, chunk_files.flatten())
    }
    | MERGE_CDR3OUT_FROM_TRUST4_CHUNKS

  if (params.keep_intermediate >= 4) {
    TRUST4.out.airr_tsv
      .groupTuple(by: [0, 1, 2, 3])
      .map { sample, isotype, has_header, filename, chunk_files ->
        tuple(sample, isotype, has_header, filename, chunk_files.flatten())
      }
      | MERGE_AIRRTSV_FROM_TRUST4_CHUNKS

    TRUST4.out.assembled_reads_fa
      .groupTuple(by: [0, 1, 2, 3])
      .map { sample, isotype, has_header, filename, chunk_files ->
        tuple(sample, isotype, has_header, filename, chunk_files.flatten())
      }
      | MERGE_ASSEMBLEDREADSFA_FROM_TRUST4_CHUNKS

    TRUST4.out.barcode_airr_tsv
      .groupTuple(by: [0, 1, 2, 3])
      .map { sample, isotype, has_header, filename, chunk_files ->
        tuple(sample, isotype, has_header, filename, chunk_files.flatten())
      }
      | MERGE_BARCODEAIRRTSV_FROM_TRUST4_CHUNKS

    TRUST4.out.barcode_report_tsv
      .groupTuple(by: [0, 1, 2, 3])
      .map { sample, isotype, has_header, filename, chunk_files ->
        tuple(sample, isotype, has_header, filename, chunk_files.flatten())
      }
      | MERGE_BARCODEREPORTTSV_FROM_TRUST4_CHUNKS

    TRUST4.out.final_out
      .groupTuple(by: [0, 1, 2, 3])
      .map { sample, isotype, has_header, filename, chunk_files ->
        tuple(sample, isotype, has_header, filename, chunk_files.flatten())
      }
      | MERGE_FINALOUT_FROM_TRUST4_CHUNKS

    TRUST4.out.raw_out
      .groupTuple(by: [0, 1, 2, 3])
      .map { sample, isotype, has_header, filename, chunk_files ->
        tuple(sample, isotype, has_header, filename, chunk_files.flatten())
      }
      | MERGE_RAWOUT_FROM_TRUST4_CHUNKS

    TRUST4.out.report_tsv
      .groupTuple(by: [0, 1, 2, 3])
      .map { sample, isotype, has_header, filename, chunk_files ->
        tuple(sample, isotype, has_header, filename, chunk_files.flatten())
      }
      | MERGE_REPORTTSV_FROM_TRUST4_CHUNKS

    TRUST4.out.toassemble_1_fq
      .groupTuple(by: [0, 1, 2, 3])
      .map { sample, isotype, has_header, filename, chunk_files ->
        tuple(sample, isotype, has_header, filename, chunk_files.flatten())
      }
      | MERGE_TOASSEMBLE1FQ_TRUST4_CHUNKS

    TRUST4.out.toassemble_2_fq
      .groupTuple(by: [0, 1, 2, 3])
      .map { sample, isotype, has_header, filename, chunk_files ->
        tuple(sample, isotype, has_header, filename, chunk_files.flatten())
      }
      | MERGE_TOASSEMBLE2FQ_FROM_TRUST4_CHUNKS

    TRUST4.out.toassemble_bc_fa
      .groupTuple(by: [0, 1, 2, 3])
      .map { sample, isotype, has_header, filename, chunk_files ->
        tuple(sample, isotype, has_header, filename, chunk_files.flatten())
      }
      | MERGE_TOASSEMBLEFA_TRUST4_CHUNKS
  }

  annot_fa
    .join(airr_align_tsv, by: [0, 1])
    .join(cdr3_out, by: [0, 1])
    | GENERATE_FINAL_REPORTS

  GENERATE_FINAL_REPORTS.out
    .map {
      sample = it[0]
      isotype = it[1]
      tuple(sample, isotype)
    }
    .collectFile(keepHeader: true, sort: true, storeDir: "${params.outdir}/umi_stats") {
      sample = it[0]
      isotype = it[1]
      ["umi_cutoffs.template.csv", "sample,isotype,umi_cutoff\n${sample},${isotype},\n"]
    }
}
