if (params.platform == "linux" || params.platform == "linux2") {
  trust4_dependency = "conda-forge::libgcc-ng==14.1.0"
} else {
  trust4_dependency = ""
}

process GENERATE_FINAL_REPORTS {
  conda "bioconda::trust4=${params.trust4_version} conda-forge::python=3.12.5 ${trust4_dependency}"
  publishDir "${params.outdir}/reports/${sample}/${isotype}", mode: "copy"
  tag "Sample:$sample Isotype:$isotype"

  input:
  tuple val(sample), val(isotype), path(annot_fa), path(airr_align_tsv), path(cdr3_out)

  output:
  tuple val(sample), val(isotype), path("TRUST4_*.tsv"), path("*.immune_viewer_*.csv"), path(".command.*")

  shell:
  if (params.use_umis && params.kit_uses_umi) {
    '''
    trust-barcoderep.pl !{cdr3_out} -a !{annot_fa} --chainsInBarcode 1 > TRUST4_barcode_report.tsv
    trust-simplerep.pl !{cdr3_out} --barcodeCnt --filterBarcoderepReadCnt !{params.umi_cutoff} --filterBarcoderep TRUST4_barcode_report.tsv > TRUST4_report.UMI_!{params.umi_cutoff}.tsv
    trust-airr.pl TRUST4_report.UMI_!{params.umi_cutoff}.tsv !{annot_fa} --airr-align !{airr_align_tsv} > TRUST4_airr.UMI_!{params.umi_cutoff}.tsv
    trust-airr.pl TRUST4_barcode_report.tsv !{annot_fa} --format barcoderep --airr-align !{airr_align_tsv} > TRUST4_barcode_airr.tsv
    python !{projectDir}/share/scripts/genImmuneViewerReports.py TRUST4_airr.UMI_!{params.umi_cutoff}.tsv --species !{params.species_common} --sampleID !{sample} --isotype !{isotype} --outdir . --umi_cutoff !{params.umi_cutoff}
    IV_REPORT_FILENAME=!{sample}_!{isotype}.UMI_!{params.umi_cutoff}.immune_viewer_report.csv
    head -n1 $IV_REPORT_FILENAME > $IV_REPORT_FILENAME.tmp
    tail -n+2 $IV_REPORT_FILENAME |  sort -t, -nr -k 3,3 >> $IV_REPORT_FILENAME.tmp
    mv $IV_REPORT_FILENAME.tmp $IV_REPORT_FILENAME
    '''
  } else {
    """
    trust-simplerep.pl ${cdr3_out} > TRUST4_report.tsv
    trust-airr.pl TRUST4_report.tsv ${annot_fa} --airr-align ${airr_align_tsv} > TRUST4_airr.tsv
    python ${projectDir}/share/scripts/genImmuneViewerReports.py TRUST4_airr.tsv --species ${params.species_common} --sampleID ${sample} --isotype ${isotype} --outdir .
    """
  }
}
