if (params.platform == "linux" || params.platform == "linux2") {
  trust4_dependency = "conda-forge::libgcc-ng==14.1.0"
} else {
  trust4_dependency = ""
}

process GENERATE_FINAL_REPORTS_WITH_UMI_CUTOFF_POST_ANALYSIS {
  conda "bioconda::trust4=${params.trust4_version} conda-forge::python=3.12.5 ${trust4_dependency}"
  publishDir "${params.analysisDir}/reports/${sample}/${isotype}", mode: "copy"
  tag "Sample:$sample Isotype:$isotype"

  input:
  tuple val(sample), val(isotype), val(umi_cutoff)

  output:
  tuple val(sample), val(isotype), path("TRUST4_*.tsv"), path("*.immune_viewer_*.csv"), path(".command.*")

  shell:
  abs_path_to_analysis_dir = file(params.analysisDir)
  '''
  TRUST4_DIR=!{abs_path_to_analysis_dir}/trust4/!{sample}/!{isotype}
  REPORT_DIR=!{abs_path_to_analysis_dir}/reports/!{sample}/!{isotype}
  TRUST4_BARCODE_REPORT=$REPORT_DIR/TRUST4_barcode_report.tsv
  ANNOT_FA=$TRUST4_DIR/TRUST4_annot.fa
  CDR3_OUT=$TRUST4_DIR/TRUST4_cdr3.out
  AIRR_ALIGN_TSV=$TRUST4_DIR/TRUST4_airr_align.tsv
  trust-simplerep.pl $CDR3_OUT --barcodeCnt --filterBarcoderepReadCnt !{umi_cutoff} --filterBarcoderep $TRUST4_BARCODE_REPORT > TRUST4_report.UMI_!{umi_cutoff}.tsv
  trust-airr.pl TRUST4_report.UMI_!{umi_cutoff}.tsv $ANNOT_FA --airr-align $AIRR_ALIGN_TSV > TRUST4_airr.UMI_!{umi_cutoff}.tsv
  python !{projectDir}/share/scripts/genImmuneViewerReports.py TRUST4_airr.UMI_!{umi_cutoff}.tsv --species !{params.species_common} --sampleID !{sample} --isotype !{isotype} --outdir . --umi_cutoff !{umi_cutoff}
  IV_REPORT_FILENAME=!{sample}_!{isotype}.UMI_!{umi_cutoff}.immune_viewer_report.csv
  head -n1 $IV_REPORT_FILENAME > $IV_REPORT_FILENAME.tmp
  tail -n+2 $IV_REPORT_FILENAME |  sort -t, -nr -k 3,3 >> $IV_REPORT_FILENAME.tmp
  mv $IV_REPORT_FILENAME.tmp $IV_REPORT_FILENAME
  '''
}
