if (params.keep_intermediate >= 1) {
  publishdir_enabled = true
} else {
  publishdir_enabled = false
}

process PREPROCESS {
  conda "${projectDir}/share/config/env/preprocess.yaml"
  publishDir "${params.outdir}/${task.process.toLowerCase()}/${sample}", mode: "copy", enabled: publishdir_enabled
  tag "Sample:$sample"

  input:
  tuple val(sample), val(meta_file)

  output:
  tuple val(sample), path("*.fastq.gz"), path(".command.*"), emit: fastq
  path("*.QC_stats.csv"), emit: qc_stats
  path("*.QC_stats.airr.csv"), emit: qc_stats_airr

  shell:
  if (params.linker_correction) {
    linker_correction_param = "--linker_correction"
  } else {
    linker_correction_param = ""
  }
  '''
  python !{projectDir}/share/scripts/preprocess.py \
    -r !{params.receptor_kit} \
    -m !{meta_file} \
    -s !{params.species_common} \
    --sample !{sample} \
    !{linker_correction_param} \
    --config_path !{projectDir}/share/config

  find . -name '*.fastq' -size +0c -print0 | xargs -0 gzip -f
  '''
}
