if (params.keep_intermediate >= 3) {
  publishdir_enabled = true
} else {
  publishdir_enabled = false
}

process SPLIT_FASTQ_FILES {
  conda "${projectDir}/share/config/env/split_fastq_files.yaml"
  publishDir "${params.outdir}/${task.process.toLowerCase()}/${sample}/${isotype}", mode: "copy", enabled: publishdir_enabled
  tag "Sample:$sample Isotype:$isotype"

  input:
  tuple val(sample), val(isotype), path(fastq1), path(fastq2)

  output:
  tuple val(sample), val(isotype), path("*.fastq.gz"), path(".command.*"), emit: fastq
  tuple val(sample), path("umi_group_sizes_frequency.*.csv"), emit: umi_group_sizes_frequency, optional: true

  shell:
  if (params.use_umis && params.kit_uses_umi && params.chunks <= 1) {
    '''
    gunzip -f -c !{fastq1} > r1.fastq

    python !{projectDir}/share/scripts/umig-aware-fastq-splitter.py \
      --outdir . \
      --fastq1 r1.fastq \
      --fastq2 r2.fastq \
      --chunks !{params.chunks} \
      --isotype !{isotype}

    ln -s !{fastq1} chunk_0_R1.fastq.gz
    ln -s !{fastq2} chunk_0_R2.fastq.gz
    '''
  } else if (params.use_umis && params.kit_uses_umi) {
    '''
    gunzip -f -c !{fastq1} > r1.fastq
    gunzip -f -c !{fastq2} > r2.fastq

    python !{projectDir}/share/scripts/umig-aware-fastq-splitter.py \
      --outdir . \
      --fastq1 r1.fastq \
      --fastq2 r2.fastq \
      --chunks !{params.chunks} \
      --isotype !{isotype}

    find . -name 'chunk_*.fastq' -size +0c -print0 | xargs -0 gzip -f
    '''
  } else if (params.chunks <= 1) {
    '''
    ln -s !{fastq1} chunk_0_R1.fastq.gz
    ln -s !{fastq2} chunk_0_R2.fastq.gz
    '''
  } else {
    '''
    gunzip -f -c !{fastq1} > r1.fastq
    gunzip -f -c !{fastq2} > r2.fastq

    python !{projectDir}/share/scripts/fastq-splitter.py \
      --outdir . \
      --fastq1 r1.fastq \
      --fastq2 r2.fastq \
      --chunks !{params.chunks}

    find . -name 'chunk_*.fastq' -size +0c -print0 | xargs -0 gzip -f
    '''
  }
}
