if (params.keep_intermediate >= 2) {
  publishdir_enabled = true
} else {
  publishdir_enabled = false
}

if (params.platform == "linux" || params.platform == "linux2") {
  trust4_dependency = "conda-forge::libgcc-ng==14.1.0"
} else {
  trust4_dependency = ""
}

process TRUST4 {
  conda "bioconda::trust4=${params.trust4_version} ${trust4_dependency}"
  publishDir "${params.outdir}/${task.process.toLowerCase()}/${sample}/${isotype}/chunks", enabled: publishdir_enabled
  errorStrategy { sleep(Math.pow(2, task.attempt) * 1000 as long); return 'retry' }
  maxRetries 10
  tag "Sample:$sample Isotype:$isotype Chunk:$chunk"

  input:
  tuple val(sample), val(isotype), val(chunk), path(fastq1), path(fastq2), path(species_fa)

  output:
  tuple val(sample), val(isotype), val("true"), val("TRUST4_airr.tsv"), path("TRUST_*_R1_airr.tsv"), emit: airr_tsv
  tuple val(sample), val(isotype), val("false"), val("TRUST4_airr_align.tsv"), path("TRUST_*_R1_airr_align.tsv"), emit: airr_align_tsv
  tuple val(sample), val(isotype), val("false"), val("TRUST4_annot.fa"), path("TRUST_*_R1_annot.fa"), emit: annot_fa
  tuple val(sample), val(isotype), val("false"), val("TRUST4_assembled_reads.fa"), path("TRUST_*_R1_assembled_reads.fa"), emit: assembled_reads_fa
  tuple val(sample), val(isotype), val("true"), val("TRUST4_barcode_airr.tsv"), path("TRUST_*_R1_barcode_airr.tsv"), emit: barcode_airr_tsv, optional: true
  tuple val(sample), val(isotype), val("true"), val("TRUST4_barcode_report.tsv"), path("TRUST_*_R1_barcode_report.tsv"), emit: barcode_report_tsv, optional: true
  tuple val(sample), val(isotype), val("false"), val("TRUST4_cdr3.out"), path("TRUST_*_R1_cdr3.out"), emit: cdr3_out
  tuple val(sample), val(isotype), val("false"), val("TRUST4_final.out"), path("TRUST_*_R1_final.out"), emit: final_out
  tuple val(sample), val(isotype), val("false"), val("TRUST4_raw.out"), path("TRUST_*_R1_raw.out"), emit: raw_out
  tuple val(sample), val(isotype), val("true"), val("TRUST4_report.tsv"), path("TRUST_*_R1_report.tsv"), emit: report_tsv
  tuple val(sample), val(isotype), val("false"), val("TRUST4_toassemble_1.fq"), path("TRUST_*_R1_toassemble_1.fq"), emit: toassemble_1_fq
  tuple val(sample), val(isotype), val("false"), val("TRUST4_toassemble_2.fq"), path("TRUST_*_R1_toassemble_2.fq"), emit: toassemble_2_fq
  tuple val(sample), val(isotype), val("false"), val("TRUST4_toassemble_bc.fa"), path("TRUST_*_R1_toassemble_bc.fa"), emit: toassemble_bc_fa, optional: true

  shell:
  if (params.repseq || !params.kit_uses_umi) {
    repseq_command = "--repseq"
  } else {
    repseq_command = ""
  }

  if (params.skipMateExtension) {
    skipMateExtension_command = "--skipMateExtension"
  } else {
    skipMateExtension_command = ""
  }
  
  if (params.unusual_gaps) {
    imgtAdditionalGap_command = "--imgtAdditionalGap ${params.unusual_gaps}"
  } else {
    imgtAdditionalGap_command = ""
  }
  
  if (params.use_umis && params.kit_uses_umi) {
    '''
    run-trust4 -1 !{fastq1} -2 !{fastq2} -f !{species_fa} \
      -t !{params.threads} \
      !{repseq_command} \
      !{skipMateExtension_command} \
      !{imgtAdditionalGap_command} \
      --readFormat r1:0:-1,r2:30:-1,bc:12:23 \
      --barcode !{fastq2} \
      --barcodeLevel molecule
    '''
  } else {
    '''
    run-trust4 -1 !{fastq1} -2 !{fastq2} -f !{species_fa} \
      -t !{params.threads} \
      !{repseq_command} \
      !{skipMateExtension_command} \
      !{imgtAdditionalGap_command}
    '''
  }
}
