import argparse
import os
import re
from collections import Counter
from itertools import islice
from pathlib import Path

from Bio import SeqIO


parser = argparse.ArgumentParser()
parser.add_argument('-o', '--outdir',
                    dest='outdir',
                    help='path to the output directory',
                    type=Path,
                    required=True)
parser.add_argument('-1', '--fastq1',
                    dest='fastq1',
                    help='path to "toassemble" R1 FASTQ file',
                    type=Path,
                    required=True)
parser.add_argument('-2', '--fastq2',
                    dest='fastq2',
                    help='path to "toassemble" R2 FASTQ file',
                    type=Path,
                    required=True)
# Need to pick a proper default for -c
parser.add_argument('-c', '--chunks',
                    default=4,
                    dest='chunks',
                    help='the number of chunks to split FASTQ files into for assembly',
                    type=int,
                    required=False)

args = parser.parse_args()

r1_fastq_in = args.fastq1
r2_fastq_in = args.fastq2
outdir = args.outdir
n_chunks = args.chunks

os.makedirs(outdir, exist_ok=True)

r1_chunk_files = [open(f'{outdir}/chunk_{i}_R1.fastq', 'w') for i in range(n_chunks)]
r2_chunk_files = [open(f'{outdir}/chunk_{i}_R2.fastq', 'w') for i in range(n_chunks)]

with open(r1_fastq_in) as r1_fastq_in_fh:
    r2_fastq_in_fh = open(r2_fastq_in)
    r2_record_generator = SeqIO.parse(r2_fastq_in_fh, 'fastq')
    for count, r1_record in enumerate(SeqIO.parse(r1_fastq_in_fh, 'fastq')):
        r2_record = next(islice(r2_record_generator, 1))
        chunk_id = count % n_chunks
        SeqIO.write(r1_record, r1_chunk_files[chunk_id], 'fastq')
        SeqIO.write(r2_record, r2_chunk_files[chunk_id], 'fastq')
