library(ggplot2)
library(purrr)
library(stringr)
library(tidyr)


params <- commandArgs(trailingOnly = TRUE)
sample_id <- params[1]
csv_file_list <- params[2:length(params)]

read_my_csv <- function(csv_file) {
  df <- read.csv(csv_file)
  df$isotype <- str_match(basename(csv_file), "umi_group_sizes_frequency.(.+).csv")[2]
  df
}
umi_group_sizes_frequency <- map_df(csv_file_list, read_my_csv)
umi_group_sizes_frequency$umis <- umi_group_sizes_frequency$umi_group_size * umi_group_sizes_frequency$umi_groups

takara_blue <- rgb(28 / 255, 130 / 255, 224 / 255)
takara_purple <- rgb(110 / 255, 0 / 255, 130 / 255)

p <- umi_group_sizes_frequency %>%
  pivot_longer(!c(umi_group_size, isotype), names_to = "count_type", values_to = "count") %>%
  ggplot(aes(umi_group_size, count, color = count_type)) +
    geom_line() +
    geom_point() +
    facet_wrap(. ~ isotype, scales = "free_y") +
    scale_x_continuous(transform = "log2") +
    labs(x = "UMI Group Size", y = "Frequency") +
    scale_color_manual(values = c(takara_purple, takara_blue), labels = c("UMI Groups", "UMIs")) +
    theme_bw()
ggsave(paste("umi_group_sizes_frequency", sample_id, "png", sep = "."), plot = p, h = 5, w = 8, units = "in")
